
% In this script we demonstarte how to perform symbolic calculations. 
clear %Clearing memory
close all

%**************************************************************************
%  Example 1: Matrix algebra (symbilic)                                   *
%**************************************************************************
syms a b c d %Declaring symbolic variables
A = [a b; c d] %Matrix
det_A = det(A)
B = A.' %Transpose
D = det(A) %Determinant
invA = inv(A) %Inverse

%**************************************************************************
%  Example 2: Solving single equation                                     *
%**************************************************************************
syms x a b c %Declaring symbolic variables
f = a*x^2 + b*x + c; %Quadratic equation
x = solve(f,x) %Using the "solve" command

%**************************************************************************
%  Example 3: Solving two equations (supply and demand)                   *
%**************************************************************************
clear
syms Q P a b c d %Declaring symbolic variables
[Qsol,Psol] = solve([Q == a - b*P, Q == -c + d*P],[Q,P])

%**************************************************************************
%  Example 4: Solving three equations                                     *
%**************************************************************************
clear
syms  x y k %Declaring symbolic variables
eq1 = x - 2*k == 0; %First equation
eq2 = y - 2*k == 0; %Second equation
eq3 = 2*x + 2*y == 100; %Third equation
eqns = [eq1,eq2,eq3];
[x,y,k] = solve(eqns,[x,y,k])

%**************************************************************************
%  Example 5: Symbolic derivatives                                        *
%**************************************************************************
clear
syms K L A a b x %Declaring symbolic variables
F = A*K^a*L^(1-a); %Defining production function
MP_K = diff(F,K) %Marginal product of capital
MP_L = diff(F,L) %Marginal product of labor


% Other examples of differentiation
f1 = diff(log(x)) %Logarithmic rule
f2 = diff(a*x^b) %Power rule
f3 = diff(a^x) %Exponetial function rule

%**************************************************************************
%  Example 6: Consumer's demand                                           *
%**************************************************************************
clear
syms  x y p_x p_y I k a %Declaring symbolic variables
u(x,y) = a*log(x) + (1-a)*log(y); %Utility function defined
L(x,y,k) = u(x,y) - k*(p_x*x + p_y*y - I); %Lagrange function defined
% k is the Lagrange multiplier)

% First Order Conditions
eq1 = diff(L,x)==0; %Partial derivative with respect to x, equal zero
eq2 = diff(L,y)==0; %Partial derivative with respect to y, equal zero
eq3 = diff(L,k)==0; %Partial derivative with respect to k, equal zero
eqns = [eq1,eq2,eq3];
% Alternatively
% eqns = gradient(L,[x,y,k])==0;

% Solving the equations
[x,y,k] = solve(eqns,[x,y,k])

% Substituting values
demand = subs([x,y],[a,p_x,p_y,I],[0.5,3,0.75,100])

% Variable Precision Arithmetic (converting to numerical)
demand = vpa(demand,32) %vpa(x,d), where x is symbolic variable, d = digits

%**************************************************************************
%  Example 7: Gradient, Hessian                                           *
%**************************************************************************
% Gradient, Hessian example 1
syms x1 x2 %Declaring symbolic variables
f(x1,x2) = 100 - x1^2 - x2^2; %Defining a function
grad1 = gradient(f(x1,x2),[x1,x2]); %Gradient vector
H1 = hessian(f(x1,x2),[x1,x2]); %Hessian matrix

disp('The function is: f(x1,x2) = 100 - x1^2 - x2^2 is:')
disp('The gradient vector is:')
grad1
disp('The Hessian matrix is:')
H1

% Gradient, Hessian example 2
syms x1 x2 x3 %Declaring symbolic variables
f(x1,x2,x3) = 2*x1^2 + x1*x2 + 4*x2^2 + x1*x3 + x3^2 + 2; %Defining a function
grad2 = gradient(f(x1,x2,x3),[x1,x2,x3]); %Gradient vector 
H2 = hessian(f(x1,x2,x3),[x1,x2,x3]); %Hessian matrix
disp('The function is f(x1,x2) = 2*x1^2 + x1*x2 + 4*x2^2 + x1*x3 + x3^2 + 2')
disp('The gradient vector is:')
grad2
disp('The Hessian matrix is:')
H2

%**************************************************************************
%  Example 8: Comparative Statics Analysis and Implicit Function Theorem  *
%**************************************************************************
syms x y a %Declaring symbolic variables (x,y endogenous, a exogenous)
F1(x,y,a) = 2*x*a + x*y + a - 2*sqrt(a) - 11; %First implicit function
F2(x,y,a) = x*y*a - 6;                       %Second implicit function
F(x,y,a) = [F1(x,y,a); F2(x,y,a)]; %Declaring symbolic system
F(3,2,1) %Evaluating the system at given the point (3,2,1)

J_xy_sym(x,y,a) = jacobian(F, [x, y]); %Jacobian w.r.t. endogenous variables
J_xy = J_xy_sym(3,2,1); %Evaluating the Jacobian at given the point (3,2,1)
det_J = det(J_xy); %Determinant of Jcobian

J_a_sym = jacobian(F,a); %Jacobian w.r.t. exogenous variable
J_a = J_a_sym(3,2,1); %Evaluating the Jacobian at given the point (3,2,1)

% Comparative statics
dxy_da = -inv(J_xy)*J_a;

%**************************************************************************
%  Example 9: Comparative Statics Analysis and Implicit Function Theorem  *
%**************************************************************************
syms x y u v %Declaring symbolic variables (x,y endogenous, u,v exogenous)
F1(x,y,u,v) = y^2 + 2*u^2 + v^2 - x*y - 15; %Declaring symbolic function
F2(x,y,u,v) = 2*y^2 + u^2 + v^2 + x*y - 38; %Declaring symbolic function
F(x,y,u,v) = [F1(x,y,u,v); F2(x,y,u,v)]; %Declaring symbolic system
F(1,4,1,-1) %Evaluating the system at given the point (1, 4, 1, -1)

J_xy_sym(x,y,u,v) = jacobian(F, [x, y]); %Jacobian w.r.t. endogenous variables
J_xy = J_xy_sym(1, 4, 1, -1); %Evaluating the Jacobian at given the point (1, 4, 1, -1)
det_J = det(J_xy); %Determinant of Jcobian

J_uv_sym = jacobian(F,[u,v]); %Jacobian w.r.t. exogenous variable
J_uv = J_uv_sym(1, 4, 1, -1); %Evaluating the Jacobian at given the point (3,2,1)

% Comparative statics
dxy_duv = -inv(J_xy)*J_uv;

